#!/usr/bin/python
# -*- coding: utf-8 -*-

# Python es un lenguaje de programación definido de tipado fuerte y
# dinámico.
# Esto quiere decir que cuando un objeto se crea, no es necesario
# definir su tipo antes, pero para operar dos objetos hace falta
# convertirlos a los tipos adecuados
# Otra caracteristica de python es que todo lo que se pueda definir
# son objetos. Esto quiere decir que tienen metodos y atributos.

a= '3'
b= 4
print b.__doc__



# Vamos a remarcar que la condicion de ser de tipado fuerte fuerza a siempre
# hacer las conversiones de objetos correspondientes (aunque hay
# excepciones como, por ejemplo, cuando utilizamos la funcion print).
# Por ejemplo, la siguiente linea da error
# print a+b
#la siguiente linea es correcta
print a+str(b)

# Comentarios de la forma ''' se incluyen un atributo llamado __doc__
# que forma parte del objeto. Tambien los modulos que se importan son
# objetos.
import sys

print sys.__doc__

# Cuando necesiteis añadir modulos o programas en otros directorios
# simplemente teneis que escribir.

sys.path.append('/tmp/')

# Tipos "tipicos" en python:
# Enteros, reales, complejos, strings,.... lo que se puede encontrar
# en cualquier lenguaje de programacion.
a= complex(4,3)
a=a+a



# Tuplas: Son listas que no se pueden modificar, pero tienen la
# ventaja de ser mas eficientes que el tipo lista normal. El
# constructor es la coma.
a=1,
print len(a)
# Las tuplas pueden almacenar distintos tipos de elementos, incluso
# estos ser tuplas.

a=('una palabra', 12, (1,2))

# Para acceder a un elemento de la tupla, simplemente tenemos que
# utilizar el operador [].
print a[0]



# Como objetos que son las tuplas, tienen attributos y metodos, estos
# se pueden constar utilizando la siguiente funcion

print dir(a)

# Los objetos que estan en una tupla no pueden ser cambiados, pero a
# cambio se pueden utilizar de una forma mas eficiente y ocupan menos
# memoria. Tambien es interesante como comprobar si un objeto esta en
# la lista o no y eso se hace de la siguiente forma.

print 1 in a # devuelve un boleano, si 1 pertenece a la lista

# Quiero hacer notar que la asignacion en python utiliza las
# tuplas. Pongamos el ejemplo de querer dar valores a dos variables.

a,b=2,3


# Similar a las tuplas, tenemos las listas, en este caso el
# constructor es [] o list

a=list((1,2))
b=[1,2]

# Las listas tienen muchas funcionalidad, como por ejemplo, poder
# utilizar los indices de una forma muy particular. Para elegir el
# ultimo elemento de la lista se puede utilizar -1.

b[-1]

# Tambien podemos utilizar, el slicing, que consiste en definir partes
# de una lista definiendo el principio y fin de la sublista

b=[0,1,2,3,4,5,6,7]

print b[2:5]

# Ahora, volvamos un momento para comentar como se realiza la
# comparacion en python. La comparacion en python se hace mediante dos
# operadores diferentes:
# is y ==. Mientras que  is comprueba si dos objetos son el mismo, ==
# compara el contenido del objeto.
a = [1]
b=  [1]
print a is b, a == b
# Ahora podemos comprobar cuanta memoria usa un objeto.

print sys.getsizeof(a)

# Existen formas de definir las listas
# iteractivamente. Esto se llama definir listas por compresion. Veamos un
# ejemplo, con la funcion range, que devuelve una lista del cero al
# numero que se le pasa en el argumento menos uno. Hagamos una lista que genera
# los 5 primeros numeros naturales al cuadrado.

a=[i**2 for i in range(5)]
print a

# Hagamos otro ejemplo, generemos los numeros cuadrados pero solo
# aquellos que son pares.

a=[i**2 for i in range(5) if i**2%2==0]
print a

# Hagamos las listas de listas de dos numeros, donde estos dos numeros
# estan entre 0 y 4.

a=[[i,j] for i in range(5) for j in range(5)]

# De la misma forma se pueden definir conjuntos:
a={ i for i in range(5)}

# y diccionarios

a={i:j for i in range(5) for j in range(5) }

# mencionemos rapidamente que los valores que se utilizan en un
# diccionario tienen que ser inmutables. Esto es que no pueden ser
# cambiados. Las listas no son objetos inmutables, por ello no pueden
# ser utilzadas como claves.

# Tanto en las listas, tuplas o conjunto se puede testar si un elemento esta
# dentro de una lista simplemente utilizando el operador in

a={i for i in range(5)}

print 1 in a


# Python tiene un particularidad muy interesante, y es que la
# tabulacion indica el inicio y fin de bloques. Tanto si estamos
# escribiendo un metodo, un bucle for o una sentencia if, el principio
# final nos lo marca  el identado del codigo.
# Veamos aqui un ejemplo. Recorramos una lista y veamos cual es la salida.

for i in range(4):
    print 1
print 2

# Siempre que utilizamos python, recomiendan utilizar la identacion de
# 4 espacios. Hay ocasiones donde esto hace que el codigo sea mas
# dificil de leer  porque la linea
# es muy larga. En esos casos se puede utilizar \ para los strings y
# dar un salto de linea.
for i in range(4):
    print "estoy imprimiendo esta linea\
           taaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaan\
           larga porque si"

# Cuando hay parentesis involucrados tambien nos podemos despreocupar
# de los saltos de linea. Por eso se recomienda identar de un modo
# adecuado.

a=(1,2,3,4,5,
   3,4,5,6,6)

# Hablemos de los bloques for. Un bloque for permite recorrer los
# objetos definidos dentro de un objeto iterable. Conjuntos, listas,
# tuplas, strings y diccionarios son iterables. Los objetos
# iterables tambien son llamados secuencias.

for letra in "una frase":
    print letra

# Cuando se itera sobre un diccionario, se esta iteranto sobre las
# claves de ese diccionario.

for elemento in {1:2,3:4}:
    print elemento


# Pasemos a hablar de los bloques if. Estos se definen de la misma
# forma, utilizando la indentacion. Veamos un ejemplo que combina un
# bloque for y un bloque if

for i in range(4):
    if i==2:
        print "dos"
    elif i>=3:
        print "tres"
    else:
        j=1

# El uso de tuplas hace mas claro el uso de los bloques if. Veamos dos
# ejemplos
a=1

if a==1 or a==2 or a==3:
    print a

# Una mejor forma de hacerlo

if a in (1,2,3):
    print a

# Como colofon, acabemos de comentar dos primitivas que son muy
# interesantes: all y any. Cuando queremos comprobar si los valores de
# una lista son todos pares podemos realizarlo de dos formas.

a=[2,4,6,8]
for i in a:
    if i%2==1:
        print "falso"
        break

# Codigo bastante feo. Se puede hacer mejor de la siguiente forma.
if any([i %2==0 for i in a]):
    print "falso"

# La funcion all comprueba que todos las relaciones se
# cumplen. Ademas, esta bien implementado para no seguir comprobando
# el resto de las condiciones si alguna de ellas es falsa.
if all([i %2==0 for i in a]):
    print "verdadero"





# Notese que el indentado juega el papel de las llaves en C por
# ejemplo. Para definir el bloque de una funcion tenemos la misma
# estructura. Para indicar que hay que definir una funcion, primero es
# necesario utilizar la palabra def, seguida del nombre de la funcion
# y entre parentesis los argumentos que tiene.

def f(x):
    print x

# para evaluar la funcion, simplemente hay que poner parentesis, con
# el argumento que necesitemos.
f(1)
# Los funciones son objetos, por lo tanto, tiene sentido de definir
# una funcion dentro de otra funcion. La siguiente funcion toma un
# valor entero y devuelve una funcion que, tiene un argumento y que
# suma a ese argumento el valor pasado por parametro.

def f(x):
    def g(y):
        return x+y
    return g
print f(3)(5)

# Las funciones tambien pueden tener varios argumentos, y a algunos de
# ellos se les puede dar un valor por defecto.

def devolver_frase(persona1, persona2='domingo'):
    return 'Hola, %s soy %s'%(persona1, persona2)

print devolver_frase("don pepito")
print devolver_frase("don pepito","don jose")

# Tambien podemos utilizar el operador * para cuando tenemos una lista
# con los argumentos.
args=("don pepito","don jose")
print devolver_frase(*args)


# Hablemos finalmente de los bloques de excepciones. Como en Java,
# tenemos que los bloques de excepciones se capturan con un bloque
# try/except
#

while True:
    try:
        x = int(raw_input("Escribe un numero: "))
        break
    except ValueError:
        print "No has escrito un numero"

# Para generar una excepcion, se puede utilizar la orden raise con la
# excepcion a utilizar. Por ejemplo, aqui vamos a utilzar una
# excepcion TypeError. Comentarla para poder seguir ejecutando el codigo

raise TypeError("hola")

# Empecemos ahora a definir clases en python. Las clases en python se
# definen mediante la palabra class. El "pseudo constructor" de una
# clase es un metodo llamado __init__ .

class Humano:
    ''' Comentario de la clase '''

    def __init__(self, nombre):
        self.nombre = nombre

# Vemos aqui que el metodo init tiene un parametro que juega aqui el
# papel de un puntero al objeto. Este mecanismo es necesario para
# poder acceder a los atributos y metodos del objeto diferenciando un
# atributo del objeto llamado nombre de una variable llamada
# nombre.

yo = Humano("Domingo")


# Metodos avanzados de diccionarios. Repasemos los diferentes metodos
# de los diccionarios y que luego seran muy utiles para la
# programacion. Recordemos que los diccionarios se pueden generar por
# compresion.
a={i:2*i for i in range(5)}

# Devolver el valor de la llave 3, se puede realizar de dos formas.

print a[3], a.get(3)

# Asignar un valor a una llave, se realiza de una forma muy similar

a[3]=10

# Utilizar el metodo get permite definir un valor por defecto. Esto
# es, si quisieramos devolver el valor 0 si la clave no esta
# definida. Supongamos que en una lista queremos contar los elementos
# y las veces que estan repetidos.

b=[1,2,3,4,3,2,1,1,3,2,1]
a={}
for elemento in b:
    a[elemento]=a.get(elemento,0)+1

print a

# En caso general, si la clave no aparecia en el diccionario, hubiera
# sido necesario escribir un bloque if. Para comprobar si una clave
# existe en el diccionario, la forma mas sencilla es utilizar la
# sentencia in

if 3 in a:
    print "3 si esta en a"


# Acabamos esta parte tecnica con la lectura de ficheros. Los objetos
# que se encargan de la lectura de ficheros son del tipo file. El
# constructor admite argumentos, tanto el modo de apertura del fichero
# como la codificacion....Para esta prueba se necesita crear un
# fichero de texto que se llame prueba.

f=open('prueba.txt','r')

print f.tell() #Posicion del fichero que estoy leyendo

#Para moverse por el fichero se puede utilizar el metodo seek.
#seek siempre tiene dos parametros, el significado del primero depende
#del segundo. El valor del segundo parametro puede ser:
#   0, para indicar
#      que el primer valor es la posicion relativa tomada desde el
#      comienzo en bytes.
#   1, para indicar que el primer valor es la posicion relativa desde
#      la situacion actual.
#   2, para indicar que el primer valor es la posicion relativa desde
#      el final

f.seek(1,0)
f.tell()
# read sin parametros lee el fichero de una vez.

print f.read()

# Escribir en ficheros tambien es sencillo, se utiliza el metodo open
# pero esta vez en modo escritura.

f=open('prueba.txt','w')

f.write("\n En este momento hemos destruido el contenido del\
fichero\n")

f.close()

# Tambien se permite abrir el fichero en modo escritura y seguir
# escribiendo desde el final del fichero.

f=open('prueba.txt', 'a')
f.write("Sigo poniendo cosas al final del fichero")
f.close()

# Mencionar, por lo extendido de su uso, la siguiente forma de abrir
# un fichero en modo lectura/escritura

with open('prueba.txt','a') as f:
    f.write("Pongo alguna cosilla mas")

# Utilizando with no hay que preocuparse por las excepciones ni por
# cerrar el fichero.
